@@ -1,7 +1,7 @@
-{-# LANGUAGE CPP                  #-}
-{-# LANGUAGE FlexibleInstances    #-}
-{-# LANGUAGE TemplateHaskell      #-}
-{-# LANGUAGE TypeFamilies         #-}
+{-# LANGUAGE CPP               #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE TemplateHaskell   #-}
+{-# LANGUAGE TypeFamilies      #-}
 
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
@@ -75,6 +75,7 @@ module Diagrams.Backend.Cairo.CmdLine
         -- * GIF support
 
         , GifOpts(..)
+        , gifRender
 
         -- * Backend tokens
 
@@ -82,63 +83,32 @@ module Diagrams.Backend.Cairo.CmdLine
        , B
        ) where
 
-import Codec.Picture
-import Codec.Picture.ColorQuant            (defaultPaletteOptions)
-import Data.Vector.Storable                (unsafeFromForeignPtr0)
-import Foreign.ForeignPtr.Safe             (ForeignPtr)
-import qualified Data.ByteString.Lazy as L (ByteString, writeFile)
-import Data.Word                           (Word8)
-import Options.Applicative
+import           Codec.Picture
+import           Codec.Picture.ColorQuant        (defaultPaletteOptions)
+import qualified Data.ByteString.Lazy            as L (ByteString, writeFile)
+import           Data.Vector.Storable            (unsafeFromForeignPtr0)
+import           Data.Word                       (Word8)
+import           Options.Applicative
 
-import Control.Lens                        ((^.), Lens', makeLenses)
-
-import Diagrams.Prelude hiding             (width, height, interval, (<>)
-                                           ,option)
-import Diagrams.Backend.Cairo
-import Diagrams.Backend.Cairo.Ptr          (renderForeignPtrOpaque)
-import Diagrams.Backend.CmdLine
+import           Diagrams.Backend.Cairo
+import           Diagrams.Backend.Cairo.Ptr      (renderForeignPtrOpaque)
+import           Diagrams.Backend.CmdLine
+import           Diagrams.Prelude                hiding (height, interval,
+                                                  option, output, width, (<>))
 
 -- Below hack is needed because GHC 7.0.x has a bug regarding export
 -- of data family constructors; see comments in Diagrams.Backend.Cairo
 #if __GLASGOW_HASKELL__ < 702 || __GLASGOW_HASKELL__ >= 704
-import Diagrams.Backend.Cairo.Internal
+import           Diagrams.Backend.Cairo.Internal
 #endif
 
-#if __GLASGOW_HASKELL__ < 706
-import Prelude hiding      (catch)
+#if __GLASGOW_HASKELL__ < 710
+import           Foreign.ForeignPtr.Safe         (ForeignPtr)
 #else
-import Prelude
+import           Foreign.ForeignPtr              (ForeignPtr)
 #endif
 
-import Data.List.Split
-
-#ifdef CMDLINELOOP
-import Data.Maybe          (fromMaybe)
-import Control.Monad       (when, mplus)
-import Control.Lens        (_1)
-
-import System.Environment  (getArgs, getProgName)
-import System.Directory    (getModificationTime)
-import System.Process      (runProcess, waitForProcess)
-import System.IO           (openFile, hClose, IOMode(..),
-                            hSetBuffering, BufferMode(..), stdout)
-import System.Exit         (ExitCode(..))
-import Control.Concurrent  (threadDelay)
-import Control.Exception   (catch, SomeException(..), bracket)
-
-import System.Posix.Process (executeFile)
-#if MIN_VERSION_directory(1,2,0)
-import Data.Time.Clock (UTCTime,getCurrentTime)
-type ModuleTime = UTCTime
-getModuleTime :: IO  ModuleTime
-getModuleTime = getCurrentTime
-#else
-import System.Time         (ClockTime, getClockTime)
-type ModuleTime = ClockTime
-getModuleTime :: IO  ModuleTime
-getModuleTime = getClockTime
-#endif
-#endif
+import           Data.List.Split
 
 -- $mainwith
 -- The 'mainWith' method unifies all of the other forms of @main@ and is now
@@ -149,7 +119,7 @@ getModuleTime = getClockTime
 -- will produce a program that looks for additional number and color arguments.
 --
 -- > ... definitions ...
--- > f :: Int -> Colour Double -> Diagram Cairo R2
+-- > f :: Int -> Colour Double -> Diagram Cairo
 -- > f i c = ...
 -- >
 -- > main = mainWith f
@@ -211,30 +181,15 @@ getModuleTime = getClockTime
 -- $ ./MyDiagram -o dia.pdf -h 200 -w 200 -l -i 10
 -- @
 
-defaultMain :: Diagram Cairo R2 -> IO ()
+defaultMain :: QDiagram Cairo V2 Double Any -> IO ()
 defaultMain = mainWith
 
-#ifdef CMDLINELOOP
-output' :: Lens' (MainOpts (Diagram Cairo R2)) FilePath
-output' = _1 . output
+instance Mainable (QDiagram Cairo V2 Double Any) where
+    type MainOpts (QDiagram Cairo V2 Double Any) = (DiagramOpts, DiagramLoopOpts)
 
-instance Mainable (Diagram Cairo R2) where
-    type MainOpts (Diagram Cairo R2) = (DiagramOpts, DiagramLoopOpts)
+    mainRender (opts, l) d = chooseRender opts d >> defaultLoopRender l
 
-    mainRender (opts,loopOpts) d = do
-        chooseRender opts d
-        when (loopOpts^.loop) (waitForChange Nothing loopOpts)
-#else
-output' :: Lens' (MainOpts (Diagram Cairo R2)) FilePath
-output' = output
-
-instance Mainable (Diagram Cairo R2) where
-    type MainOpts (Diagram Cairo R2) = DiagramOpts
-
-    mainRender opts d = chooseRender opts d
-#endif
-
-chooseRender :: DiagramOpts -> Diagram Cairo R2 -> IO ()
+chooseRender :: DiagramOpts -> QDiagram Cairo V2 Double Any -> IO ()
 chooseRender opts d =
   case splitOn "." (opts ^. output) of
     [""] -> putStrLn "No output file given."
@@ -249,9 +204,9 @@ chooseRender opts d =
                    Cairo
                    ( CairoOptions
                      (opts^.output)
-                     (mkSizeSpec
-                       (fromIntegral <$> opts ^. width )
-                       (fromIntegral <$> opts ^. height)
+                     (fromIntegral <$> mkSizeSpec2D
+                       (opts ^. width )
+                       (opts ^. height)
                      )
                      outTy
                      False
@@ -278,12 +233,12 @@ chooseRender opts d =
 -- $ ./MultiTest --selection bar -o Bar.png -w 200
 -- @
 
-multiMain :: [(String, Diagram Cairo R2)] -> IO ()
+multiMain :: [(String, QDiagram Cairo V2 Double Any)] -> IO ()
 multiMain = mainWith
 
-instance Mainable [(String, Diagram Cairo R2)] where
-    type MainOpts [(String, Diagram Cairo R2)]
-        = (MainOpts (Diagram Cairo R2), DiagramMultiOpts)
+instance Mainable [(String, QDiagram Cairo V2 Double Any)] where
+    type MainOpts [(String, QDiagram Cairo V2 Double Any)]
+        = (MainOpts (QDiagram Cairo V2 Double Any), DiagramMultiOpts)
 
     mainRender = defaultMultiMainRender
 
@@ -303,62 +258,13 @@ instance Mainable [(String, Diagram Cairo R2)] where
 --
 -- The @--fpu@ option can be used to control how many frames will be
 -- output for each second (unit time) of animation.
-animMain :: Animation Cairo R2 -> IO ()
+animMain :: Animation Cairo V2 Double -> IO ()
 animMain = mainWith
 
-instance Mainable (Animation Cairo R2) where
-    type MainOpts (Animation Cairo R2) = (MainOpts (Diagram Cairo R2), DiagramAnimOpts)
+instance Mainable (Animation Cairo V2 Double) where
+    type MainOpts (Animation Cairo V2 Double) = ((DiagramOpts, DiagramAnimOpts), DiagramLoopOpts)
 
-    mainRender = defaultAnimMainRender output'
-
-#ifdef CMDLINELOOP
-waitForChange :: Maybe ModuleTime -> DiagramLoopOpts -> IO ()
-waitForChange lastAttempt opts = do
-    prog <- getProgName
-    args <- getArgs
-    hSetBuffering stdout NoBuffering
-    go prog args lastAttempt
-  where go prog args lastAtt = do
-          threadDelay (1000000 * opts^.interval)
-          -- putStrLn $ "Checking... (last attempt = " ++ show lastAttempt ++ ")"
-          (newBin, newAttempt) <- recompile lastAtt prog (opts^.src)
-          if newBin
-            then executeFile prog False args Nothing
-            else go prog args $ newAttempt `mplus` lastAtt
-
--- | @recompile t prog@ attempts to recompile @prog@, assuming the
---   last attempt was made at time @t@.  If @t@ is @Nothing@ assume
---   the last attempt time is the same as the modification time of the
---   binary.  If the source file modification time is later than the
---   last attempt time, then attempt to recompile, and return the time
---   of this attempt.  Otherwise (if nothing has changed since the
---   last attempt), return @Nothing@.  Also return a Bool saying
---   whether a successful recompilation happened.
-recompile :: Maybe ModuleTime -> String -> Maybe String -> IO (Bool, Maybe ModuleTime)
-recompile lastAttempt prog mSrc = do
-  let errFile = prog ++ ".errors"
-      srcFile = fromMaybe (prog ++ ".hs") mSrc
-  binT <- maybe (getModTime prog) (return . Just) lastAttempt
-  srcT <- getModTime srcFile
-  if (srcT > binT)
-    then do
-      putStr "Recompiling..."
-      status <- bracket (openFile errFile WriteMode) hClose $ \h ->
-        waitForProcess =<< runProcess "ghc" ["--make", srcFile]
-                           Nothing Nothing Nothing Nothing (Just h)
-
-      if (status /= ExitSuccess)
-        then putStrLn "" >> putStrLn (replicate 75 '-') >> readFile errFile >>= putStr
-        else putStrLn "done."
-
-      curTime <- getModuleTime
-      return (status == ExitSuccess, Just curTime)
-
-    else return (False, Nothing)
-
- where getModTime f = catch (Just <$> getModificationTime f)
-                            (\(SomeException _) -> return Nothing)
-#endif
+    mainRender (opts, l) d =  defaultAnimMainRender chooseRender output opts d >> defaultLoopRender l
 
 -- | @gifMain@ takes a list of diagram and delay time pairs and produces a
 --   command line program to generate an animated GIF, with options @GifOpts@.
@@ -386,12 +292,12 @@ recompile lastAttempt prog mSrc = do
 --    --looping-off            Turn looping off
 --    --loop-repeat ARG        Number of times to repeat
 -- @
-gifMain :: [(Diagram Cairo R2, GifDelay)] -> IO ()
+gifMain :: [(QDiagram Cairo V2 Double Any, GifDelay)] -> IO ()
 gifMain = mainWith
 
 -- | Extra options for animated GIFs.
-data GifOpts = GifOpts { _dither :: Bool
-                       , _noLooping :: Bool
+data GifOpts = GifOpts { _dither     :: Bool
+                       , _noLooping  :: Bool
                        , _loopRepeat :: Maybe Int}
 
 makeLenses ''GifOpts
@@ -412,8 +318,8 @@ instance Parseable GifOpts where
                        ( long "loop-repeat"
                       <> help "Number of times to repeat" )
 
-instance Mainable [(Diagram Cairo R2, GifDelay)] where
-    type MainOpts [(Diagram Cairo R2, GifDelay)] = (DiagramOpts, GifOpts)
+instance Mainable [(QDiagram Cairo V2 Double Any, GifDelay)] where
+    type MainOpts [(QDiagram Cairo V2 Double Any, GifDelay)] = (DiagramOpts, GifOpts)
 
     mainRender (dOpts, gOpts) ds = gifRender (dOpts, gOpts) ds
 
@@ -425,7 +331,7 @@ imageRGB8FromUnsafePtr w h ptr = pixelMap f cImg
 
 
 encodeGifAnimation' :: [GifDelay] -> GifLooping -> Bool
-                   -> [Image PixelRGB8] -> Either String (L.ByteString)
+                   -> [Image PixelRGB8] -> Either String L.ByteString
 encodeGifAnimation' delays looping dithering lst =
     encodeGifImages looping triples
       where
@@ -444,7 +350,7 @@ scaleInt i num denom
   | num == 0 || denom == 0 = i
   | otherwise = round (num / denom * fromIntegral i)
 
-gifRender :: (DiagramOpts, GifOpts) -> [(Diagram Cairo R2, GifDelay)] -> IO ()
+gifRender :: (DiagramOpts, GifOpts) -> [(QDiagram Cairo V2 Double Any, GifDelay)] -> IO ()
 gifRender (dOpts, gOpts) lst =
   case splitOn "." (dOpts^.output) of
     [""] -> putStrLn "No output file given"
@@ -461,7 +367,7 @@ gifRender (dOpts, gOpts) lst =
                                 Just n  -> LoopingRepeat (fromIntegral n)
                dias = map fst lst
                delays = map snd lst
-               (diaWidth, diaHeight) = size2D (head dias)
+               V2 diaWidth diaHeight = size (head dias)
            fPtrs <- mapM (renderForeignPtrOpaque w h) dias
            let imageRGB8s = map (imageRGB8FromUnsafePtr w h) fPtrs
                result = writeGifAnimation'
@@ -473,4 +379,4 @@ gifRender (dOpts, gOpts) lst =
            case result of
              Left s   -> putStrLn s
              Right io -> io
-       | otherwise -> putStrLn $ "File name must end with .gif"
+       | otherwise -> putStrLn "File name must end with .gif"
